#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Indigo Tuya plugin
# Copyright (c) 2023 Alan Carter. All rights reserved.
#

################################################################################
# Imports
################################################################################

import ipaddress
from tuya_iot import (
    TuyaOpenAPI,
    AuthType
)


################################################################################
# Globals
################################################################################

def updateVar(name, value, folder=0):

    if name not in indigo.variables:
        indigo.variable.create(name, value=value, folder=folder)
    else:
        indigo.variable.updateValue(name, value)

################################################################################
class Plugin(indigo.PluginBase):

    ########################################
    # Class properties
    ########################################

    def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
     
        indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
        self.userName = pluginPrefs.get("userName", "")
        self.userPass = pluginPrefs.get("userPass", "")
        self.clientId = pluginPrefs.get("clientId", "")
        self.clientSecret = pluginPrefs.get("clientSecret", "")
        self.endPoint = pluginPrefs.get("endPoint", "")
        self.logComms = pluginPrefs.get("logComms", False)
    
    ########################################
    def __del__(self):
    
        indigo.PluginBase.__del__(self)
        
    ########################################
    def startup(self):
    
        indigo.server.log("Starting Tuya plugin")
        self.openapi = TuyaOpenAPI(self.endPoint, self.clientId, self.clientSecret, AuthType.CUSTOM)
        self.openapi.connect(self.userName, self.userPass)

    def deviceStartComm(self, dev):
    
        dev.stateListOrDisplayStateIdChanged()
        return


    def shutdown(self):
    
        indigo.server.log("Stopping Tuya plugin")
        pass
  
    ########################################
    
    def runConcurrentThread(self):
    
        indigo.server.log("Starting Tuya monitoring thread")
        try:
            while True:
                self.getReadings()
                self.sleep(30)
        except self.StopThread:
            self.shutdown()


    ########################################
    # Called when user prefs are changed
    
    def closedPrefsConfigUi(self, valuesDict, userCancelled):
    
        # If the user saved preferences, update logging parameters
        if userCancelled == False:
            logComms = self.logComms
            self.logComms = valuesDict.get("logComms", False)
            if self.logComms != logComms:
                if self.logComms:
                    indigo.server.log("Comms logging is on")
                else:
                    indigo.server.log("Comms logging is off")


        
    ########################################
    # Tuya specific functions
    ########################################

    def getReadings(self):
    
        for dev in indigo.devices.iter("self"):

            # Get readings      
            response = self.openapi.get("/v1.0/iot-03/devices/{}/status".format(dev.address))
            if response["success"] == True:
                if self.logComms:
                    indigo.server.log("Device states for %s" % dev.name)
                    indigo.server.log(response)
                if dev.deviceTypeId == "tuyaTemp":
                    temperature = float(response["result"][0]["value"]) / 10
                    humidity = response["result"][1]["value"]
                    battery = response["result"][2]["value"]
                    unit = response["result"][3]["value"]      
                    key_value_list = [
                        {'key':'temperature', 'value':temperature, 'uiValue':str(temperature)+" "+unit.upper(), 'decimalPlaces':1},
                        {'key':'humidity', 'value':float(humidity), 'uiValue':str(humidity)+"%"},
                        {'key':'battery', 'value':battery}
                    ]       
                elif dev.deviceTypeId == "tuyaPlug":
                    onState = response["result"][0]["value"]
                    if onState:
                        dev.updateStateOnServer(key="onOffState", value=True, clearErrorState=True)
                        dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOn)
                    else:
                        dev.updateStateOnServer(key="onOffState", value=False, clearErrorState=True)
                        dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOff)
                    current = float(response["result"][2]["value"]) / 10
                    power = float(response["result"][3]["value"]) / 10
                    voltage = float(response["result"][4]["value"]) / 10
                    key_value_list = [
                        {'key':'current', 'value':current, 'uiValue':str(current)+" "+" A", 'decimalPlaces':1},
                        {'key':'power', 'value':float(power), 'uiValue':str(power)+" W"},
                        {'key':'voltage', 'value':float(voltage), 'uiValue':str(voltage)+" V"}
                    ]           
                dev.updateStatesOnServer(key_value_list)
            else:
                indigo.server.log("Error getting readings for Tuya device", isError=True)

            # Get info
            response = self.openapi.get("/v1.1/iot-03/devices/{}".format(dev.address))
            if response["success"] == True:
                if self.logComms:
                    indigo.server.log("Device info for %s" % dev.name)
                    indigo.server.log(response)     
                local_key = response["result"]["local_key"]
                category = response["result"]["category_name"]
                model = response["result"]["model"]
                name = response["result"]["name"]
                product = response["result"]["product_name"]
                if response["result"]["online"] == True:
                    status = "Online"
                else:
                    status = "Offline"
        
                key_value_list = [
                    {'key':'local_key', 'value':local_key},
                    {'key':'category', 'value':category},
                    {'key':'model', 'value':model},
                    {'key':'name', 'value':name},
                    {'key':'product', 'value':product},
                    {'key':'status', 'value':status},
                ]
                dev.updateStatesOnServer(key_value_list)
  
            else:
                indigo.server.log("Error getting info for Tuya device", isError=True)

        

    ########################################
    # Smart plug action callback
    ########################################
    
    # Main switch action bottleneck called by Indigo Server.
    def actionControlDevice(self, action, dev):

        if action.deviceAction == indigo.kDeviceAction.TurnOn:
            data = {"commands":[{"code":"switch","value":True}]}
            try:
                response = self.openapi.post("/v1.0/iot-03/devices/{}/commands".format(dev.address), data)
                if response["success"] == True:
                    dev.updateStateOnServer("onOffState", True)
                    dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOn)
                else:
                    indigo.server.log("%s 'on' command failed" % dev.name, isError=True)
            except:
                indigo.server.log("%s 'on' command failed" % dev.name, isError=True)
                
        elif action.deviceAction == indigo.kDeviceAction.TurnOff:
            data = {"commands":[{"code":"switch","value":False}]}
            try:
                response = self.openapi.post("/v1.0/iot-03/devices/{}/commands".format(dev.address), data)
                if response["success"] == True:
                    dev.updateStateOnServer("onOffState", False)
                    dev.updateStateImageOnServer(indigo.kStateImageSel.PowerOff)
                else:
                    indigo.server.log("%s 'off' command failed" % dev.name, isError=True)
            except:
                indigo.server.log("%s 'off' command failed" % dev.name, isError=True)

